package com.caimei.utils;

import com.caimei.model.en.QrCodeSize;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.common.BitMatrix;
import org.springframework.util.StringUtils;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2020/3/16
 */
public class ImageUtils {

	public static String getImageURL(String dirName, String src, String domain) {
		return getImageURL(dirName, src, 0,domain);
	}

	/***
	 * 获取图片地址
	 *
	 * @param src
	 *            保存在数据库中的图片文件名
	 * @param type
	 *            图片的前缀 (如 type = 200 那么则获取的图片是 200_XXX的图片)
	 * @param dirName
	 *            图片保存的文件夹名 如 (league)
	 * @param domain
	 * 			   加上域名拼成完整路径
	 * @return
	 */
	public static String getImageURL(String dirName, String src, int type, String domain) {


		//正式环境 域名 http --- https处理
		if (domain != null && !"".equals(domain) && domain.startsWith("http:") && domain.toLowerCase().lastIndexOf("config/beta")== -1 && domain.toLowerCase().lastIndexOf("localhost")== -1) {
			domain = domain.replace("http:", "https:");
		}

		//正式环境 图片地址 http --- https处理
		if(src != null && src.startsWith("https:")){
			//非正式环境 使用http
			if (domain !=null && !"".equals(domain) && domain.toLowerCase().lastIndexOf("config/beta")>-1 || domain.toLowerCase().lastIndexOf("localhost")>-1){
				src = src.replace("https:","http:");
			}
			return src;
		}

		//正式环境 图片地址  http --- https处理
		if(src != null && src.startsWith("http:")){
			//非正式环境 使用http
			if (domain !=null && !"".equals(domain) && domain.toLowerCase().lastIndexOf("config/beta")==-1 && domain.toLowerCase().lastIndexOf("localhost")==-1){
				src = src.replace("http:","https:");
			}
			return src;
		}
		type = 0 ;
		dirName = dirName.trim();
		if (dirName == null) {dirName = "";}
		if(src == null || src.equalsIgnoreCase("null")) {src = "";}
		if (src.indexOf(",") > 0) {
			String tmp = src;
			src = tmp.substring(0, tmp.indexOf(","));
		}
		String image = "/img/default/none.jpg";
		if (dirName.equals("user")) {
			image = "/img/default/HeaderImg.png";
		} else if (dirName.equals("club")) {
			image = "/img/default/default_club.jpg";
		} else if (dirName.equals("shopLogo")) {
			image = "/img/default/suppliver.jpg";
		}else if (dirName.equals("caiMeiImage")) {
			image = "/img/default/caiMeiImage.jpg";
		}else {
			image = "/img/default/none.jpg";
		}
		if (src != null && !src.equals("")) {
			if (type != 0 || dirName.equals("product")) {
				src = src.replace("\\", "/");
				String srcActual = src;
				String subDirName = "";
				int index = src.lastIndexOf("/");

				if (index != -1) {
					subDirName = src.substring(0, index + 1);
					srcActual = src.substring(index + 1);
				}
				boolean b = src.startsWith("/uploadFile");
				if(b){
					image = src;
				}else{
					image = "/uploadFile/" + dirName + "/" + subDirName
							+ (type == 0 ? "" : type + "_") + srcActual;
				}
			} else {
				boolean b = src.startsWith("/uploadFile");
				if(b){
					image = src;
				}else{
					image = "/uploadFile/" + dirName + "/" + src;
				}
			}
		}else {
			if(StringUtils.isEmpty(image)) {
				image = "/img/default/none.jpg";
			}
		}
		return domain + image;
	}

	public static String getProductImageURL(String image, int type, String domain) {
		if (image == null) {
			return getImageURL("product", "", type, domain);
		}
		return getImageURL("product", image, type, domain);
	}

	/**
	 * 添加水印图片
	 * @param waterMarkImg	水印图片
	 * @param srcImgURL		原图片路径
	 * @param degree		旋转角度
	 * @return
	 */
	public static String markImageByIcon(Image waterMarkImg, String srcImgURL, Integer degree) {
		//io流
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			//原图片
			ImageIcon imageIcon = new ImageIcon(new URL(srcImgURL));
			Image srcImg = imageIcon.getImage();
			BufferedImage buffImg = ImageIO.read(new URL(srcImgURL));

			// 1、得到画笔对象
			Graphics2D g = buffImg.createGraphics();

			// 2、设置对线段的锯齿状边缘处理
			g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
					RenderingHints.VALUE_INTERPOLATION_BILINEAR);

			g.drawImage(
					srcImg.getScaledInstance(srcImg.getWidth(null),
							srcImg.getHeight(null), Image.SCALE_SMOOTH), 0, 0,
					null);
			// 3、设置水印旋转
			if (null != degree) {
				g.rotate(Math.toRadians(degree),
						(double) buffImg.getWidth() / 2,
						(double) buffImg.getHeight() / 2);
			}


			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP,
					1f));

			// 6、水印图片的位置
			int srcWidth = srcImg.getWidth(null);
			int srcHeight = srcImg.getHeight(null);
			int waterMarkWidth = waterMarkImg.getWidth(null);
			int waterMarkHeight = waterMarkImg.getHeight(null);
			g.drawImage(waterMarkImg, (srcWidth - waterMarkWidth) / 2, srcHeight - waterMarkHeight, null);
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER));
			// 7、释放资源
			g.dispose();

			//将图片写入流中
			ImageIO.write(buffImg, "png", baos);
			//转换成字节
			byte[] bytes = baos.toByteArray();
			BASE64Encoder encoder = new BASE64Encoder();
			//转换成base64串
			String addImage = encoder.encodeBuffer(bytes).trim();
			//删除 \r\n
			addImage = addImage.replaceAll("\n", "").replaceAll("\r", "");
			return "data:image/jpg;base64," + addImage;

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				if (null != baos) {
					baos.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	public static String addQrCode(String srcImgURL, String qrCodeLink, Integer addTemplateType) {
		//io流
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			//原图片
			ImageIcon imageIcon = new ImageIcon(new URL(srcImgURL));
			Image srcImg = imageIcon.getImage();
			BufferedImage buffImg = ImageIO.read(new URL(srcImgURL));
			int srcWidth = srcImg.getWidth(null);
			int srcHeight = srcImg.getHeight(null);

			//二维码图片
			int qrCodeSize = getQrCodeSize(srcWidth);
			BufferedImage qrCodeImg = createQrCode(qrCodeLink, qrCodeSize);

			// 1、得到画笔对象
			Graphics2D g = buffImg.createGraphics();

			// 2、设置对线段的锯齿状边缘处理
			g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
					RenderingHints.VALUE_INTERPOLATION_BILINEAR);

			g.drawImage(
					srcImg.getScaledInstance(srcImg.getWidth(null),
							srcImg.getHeight(null), Image.SCALE_SMOOTH), 0, 0,
					null);
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP,
					1f));

			// 6、二维码图片的位置
			int qrCodeWidth = qrCodeImg.getWidth(null);
			int qrCodeHeight = qrCodeImg.getHeight(null);
			int x = 0;
			int y = 0;
			if (1 == addTemplateType) {
				x = srcWidth / 4 - qrCodeWidth / 2 - qrCodeWidth/3;
				y = srcHeight* 3 / 4 - qrCodeHeight / 2 - 12;
			} else if (2 == addTemplateType) {
				x = srcWidth * 3 / 4 - qrCodeWidth / 2 + qrCodeWidth;
				y = (srcHeight - qrCodeHeight) / 2;
			} else if (3 == addTemplateType) {
				x = srcWidth / 4 - qrCodeWidth / 2 - qrCodeWidth;
				y = (srcHeight - qrCodeHeight) / 2;
			}
			g.drawImage(qrCodeImg, x, y, null);
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER));
			// 7、释放资源
			g.dispose();

			//将图片写入流中
			String fileType = srcImgURL.substring(srcImgURL.lastIndexOf(".") + 1);
			ImageIO.write(buffImg, fileType, baos);
			//转换成字节
			byte[] bytes = baos.toByteArray();
			BASE64Encoder encoder = new BASE64Encoder();
			//转换成base64串
			String addImage = encoder.encodeBuffer(bytes).trim();
			//删除 \r\n
			addImage = addImage.replaceAll("\n", "").replaceAll("\r", "");
			return "data:image/jpg;base64," + addImage;

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				if (null != baos) {
					baos.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	private static int getQrCodeSize(int srcWidth) {
		int qrCodeSize = QrCodeSize.LARGE;
		if (srcWidth < 1350) {
			qrCodeSize = QrCodeSize.MEDIUM;
		}
		if (srcWidth < 1000) {
			qrCodeSize = QrCodeSize.SMALL;
		}
		return qrCodeSize;
	}

	private static final int BLACK = 0xFF000000;
	private static final int WHITE = 0xFFFFFFFF;

	public static BufferedImage createQrCode(String url,int qrCodeSize) {
		try {
			Map<EncodeHintType, String> hints = new HashMap<>();
			hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");
			//设置二维码白边的大小
			hints.put(EncodeHintType.MARGIN, "0");
			BitMatrix bitMatrix = new MultiFormatWriter().encode(url, BarcodeFormat.QR_CODE, qrCodeSize, qrCodeSize, hints);
			int width = bitMatrix.getWidth();
			int height = bitMatrix.getHeight();
			BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			for (int x = 0; x < width; x++) {
				for (int y = 0; y < height; y++) {
					image.setRGB(x, y, bitMatrix.get(x, y) ? BLACK : WHITE);
				}
			}
			return image;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
}
